from sigla import process_node
from sigla.lib.helpers.loaders import load_xml, load_string


def run(file=None, content=None, filters=None):
    if filters is None:
        filters = {}

    if file:
        root = load_xml(file)
    elif content:
        root = load_string(content)
    else:
        raise Exception("You need to prove a file or some content")

    process_node(root, filters=filters)



import importlib.machinery
import types


def load_module(module_name, module_path):
    loader = importlib.machinery.SourceFileLoader(module_name, module_path)
    module = types.ModuleType(loader.name)
    loader.exec_module(module)
    return module


def load_filters_from(module_path):
    filters = {}
    module_name = "filters"
    try:
        filters_module = load_module(module_name, module_path)
        if "FILTERS" in dir(filters_module):
            filters = filters_module.FILTERS
    except FileNotFoundError:
        pass
    return filters


from typing import Dict, Union, List
import pydash as _

from sigla.lib.helpers.Context import Context


class Node(object):
    attributes: Dict[str, Union[str, int]] = {}
    meta: Dict[str, Union[str, int]] = {}
    children: List["Node"] = []
    kind = "node"

    def is_(self, what):
        return self.kind == what

    def flatten(self):
        return _.flatten(
            [self, *list(map(lambda e: e.flatten(), self.children))]
        )

    def __init__(
        self, children: List["Node"] = None, attributes=None, meta=None
    ):
        if children is None:
            children = []
        if attributes is None:
            attributes = {}
        if meta is None:
            meta = {}

        self.attributes = attributes
        self.meta = meta
        self.children = children

    @classmethod
    def meta_from_node(cls, node):
        meta = {
            "tag": node.tag.split("-")[-1],
            "otag": node.tag,  # otag => original tag
        }
        return meta

    @classmethod
    def attributes_from_node(cls, node):
        attributes = {}
        for k, v in node.attrib.items():
            if k.endswith("-int"):
                attributes[k.replace("-int", "")] = int(v)
            else:
                attributes[k] = v
        return attributes

    def __getattribute__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError as e:
            if name in self.attributes.keys():
                return self.attributes[name]
            elif name in self.meta.keys():
                return self.meta[name]
            else:
                raise e

    def __iter__(self):
        return iter(self.children)

    def get_template_name(self):
        return self.otag

    def process(self, ctx: Context = None):
        raise NotImplementedError("Implement this method")



import pydash as _
from jinja2 import Environment, FileSystemLoader


def as_kwargs_filter(obj):
    kwargs = []
    for k, v in obj.items():
        if type(v) == int:
            kwargs.append(f"{k}={v}")
        else:
            kwargs.append(f'{k}="{v}"')
    return ", ".join(kwargs)


def map_get_filter(arr, key):
    return [_.get(o, key) for o in arr]


def without_filter(obj, *args):
    result = {}
    for k, v in obj.items():
        if k not in args:
            result[k] = v
    return result


def jinja(template: str, filters=None, **kwargs):
    if filters is None:
        filters = {}

    loader = FileSystemLoader('.sigla/templates')
    env = Environment(loader=loader)

    env.filters["without"] = without_filter
    env.filters["as_kwargs"] = as_kwargs_filter
    env.filters["map_get"] = map_get_filter

    env.filters["get_nested"] = lambda arr, field: map(
        lambda el: _.get(el, field), arr
    )

    env.filters["flatten"] = _.flatten
    env.filters["flatten_depth"] = _.flatten_depth
    env.filters["flatten_deep"] = _.flatten_deep
    env.filters["get"] = _.get
    env.filters["map"] = _.map_
    env.filters["filter"] = _.filter_
    env.filters["uniq"] = _.uniq

    for k, v in filters.items():
        env.filters[k] = v

    return env.from_string(template).render(**kwargs)



import click
import textwrap
from os.path import join
from pathlib import Path
from sigla import __version__
from sigla.cli.utils import load_filters_from
from sigla.lib.helpers.files import ensure_dirs, ensure_file
from sigla.main import run
from sigla.lib.Config import Config
from sigla.cli.SnapshotCli import SnapshotCli


class SiglaConfig(Config):
    config: Path = Path.cwd().joinpath(".sigla/config.json")
    defaults = {
        "path_templates": ".sigla/templates",
        "path_snapshots": ".sigla/snapshots",
        "path_definitions": ".sigla/definitions",
        "path_filters": ".sigla/filters.py",
    }


pass_config = click.make_pass_decorator(SiglaConfig, ensure=True)


# @click.option("--debug/--no-debug", default=False)
# click.echo('Debug mode is %s' % ('on' if debug else 'off'))
@click.group()
@pass_config
def cli(config):
    config.load()


@cli.command()
@pass_config
def init(config):
    ensure_dirs(
        config["path_templates"],
        config["path_snapshots"],
        config["path_definitions"],
    )
    DEFAULT_FILTERS_FILE_CONTENT = textwrap.dedent(
        """
        \"\"\"
        Export filters to use on the templates using the `FILTERS` variable
        \"\"\"
        import json


        def dump(var):
            return json.dumps(var, indent=4)


        FILTERS = {"dump": dump}
        """
    )

    ensure_file(config["path_filters"], DEFAULT_FILTERS_FILE_CONTENT)
    config.save()


@cli.command()
@pass_config
@click.argument("name", type=click.types.STRING)
def new_definition(config, name):
    p = Path(join(config["path_definitions"], name + ".xml"))
    p.parent.mkdir(parents=True, exist_ok=True)

    if p.exists():
        raise Exception("This definition already exists")

    name = name.replace("/", "-")

    # if file exists throw error
    with open(p, "w") as h:
        h.write(textwrap.dedent(
            f"""\
                <root>
                    <file name="output/{name}[.ext]">
                        <{name}>
                            [...]
                        </{name}>
                    </file>
                </root>
            """
        ))


@cli.command()
@pass_config
@click.argument("references", nargs=-1, type=click.types.STRING)
def run_definition(config, references):
    filters = load_filters_from(config["path_filters"])

    for reference in references:
        glob = Path(config["path_definitions"]).glob(f"{reference}.xml")
        for p in glob:
            if not p.exists():
                raise Exception("This definition does not exists")
            if p.is_dir():
                continue
            run(file=p, filters=filters)
        return


# @cli.command()  # @cli, not @click!
# @click.argument("files", nargs=-1, type=click.Path(exists=True))
# def render(files):
#     for file in files:
#         run(file=file)


@cli.command()
def version():
    """
    Prints the current version
    """
    click.echo(f"Version: {__version__}")


@cli.command()
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def snapshot(files):
    """
    Makes snapshots
    """
    for file in files:
        snap = SnapshotCli(file)
        snap.make_snapshots()


@cli.command()
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def verify(files):
    """
    Verify snapshots
    """
    for file in files:
        snap = SnapshotCli(file)
        snap.verify_snapshots()


if __name__ == "__main__":
    cli()
